import uuid

_MIN_BACKEND_VERSION = "0.16.6rc1"

_SIZE_LIMIT_BYTES = 20_971_520  # 20MB by default
_AUTO_SCALE_UP_QSIZE_TRIGGER = 200
_AUTO_SCALE_UP_NTHREADS_LIMIT = 32
_AUTO_SCALE_DOWN_NEMPTY_TRIGGER = 4
_BLOCKSIZE_BYTES = 1024 * 1024  # 1MB
_BOUNDARY = uuid.uuid4().hex
_TRACING_QUEUE_MAX_SIZE = 10_000
