# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["RunTypeEnum"]

RunTypeEnum: TypeAlias = Literal["tool", "chain", "llm", "retriever", "embedding", "prompt", "parser"]
