# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SortParamsForRunsComparisonViewParam"]


class SortParamsForRunsComparisonViewParam(TypedDict, total=False):
    sort_by: Required[str]

    sort_order: Literal["ASC", "DESC"]
