# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .example_with_runs_ch import ExampleWithRunsCh

__all__ = ["RunCreateResponse"]

RunCreateResponse: TypeAlias = List[ExampleWithRunsCh]
