# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["InsightUpdateResponse"]


class InsightUpdateResponse(BaseModel):
    """Response to update a session cluster job."""

    name: str

    status: str
